//-*-c++-*------------------------------------------------------------
//
// File name : patPythonResults.cc
// Author :    \URL[Michel Bierlaire]{http://roso.epfl.ch/mbi}
// Date :      Sat Jul 22 19:08:10 2006
//
//--------------------------------------------------------------------

#include "patDisplay.h"
#include <sstream>
#include "patPythonResults.h"

const char* patPythonResults::getTimeStamp() {
  return timeStamp.c_str() ;
}

const char* patPythonResults::getVersion() {
  return version.c_str() ;
}

const char* patPythonResults::getDescription() {
  return description.c_str() ;
}

const char* patPythonResults::getModel() {
  return model.c_str() ;
}

const char* patPythonResults::getDrawsType() {
  return drawsType.c_str() ;
}

const char* patPythonResults::getVarianceCovariance() {
  return varianceCovariance.c_str() ;
}

const char* patPythonResults::getParamName(unsigned int index) {
  if (index >= paramNames.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << paramNames.size()-1 << "]" ;
    WARNING(str.str()) ;
    return("__error") ;
  }
  return paramNames[index].c_str() ;
}

double patPythonResults::getEstimate(unsigned int index) {
  if (index >= estimates.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << estimates.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double();
  }
  return estimates[index] ;
}

double patPythonResults::getEstimate(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found = 
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknow parameter " << theName) ;
    return double() ;
  }
  return estimates[found->second] ;
}

double patPythonResults::getStdErr(unsigned int index) {
  if (index >= stdErr.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << stdErr.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return stdErr[index] ;
}

double patPythonResults::getStdErr(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return stdErr[found->second] ;
}

double patPythonResults::getTTest(unsigned int index) {
  if (index >= tTest.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << tTest.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return tTest[index] ;
}

double patPythonResults::getTTest(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return tTest[found->second] ;
}

double patPythonResults::getPValue(unsigned int index) {
  if (index >= pValue.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << pValue.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return pValue[index] ;
}

double patPythonResults::getPValue(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return pValue[found->second] ;
}

double patPythonResults::getStdErrRobust(unsigned int index) {
  if (index >= stdErrRobust.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << stdErrRobust.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return stdErrRobust[index] ;
}

double patPythonResults::getStdErrRobust(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return stdErrRobust[found->second] ;
}

double patPythonResults::getTTestRobust(unsigned int index) {
  if (index >= tTestRobust.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << tTestRobust.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return tTestRobust[index] ;
}

double patPythonResults::getTTestRobust(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return tTestRobust[found->second] ;
}

double patPythonResults::getPValueRobust(unsigned int index) {
  if (index >= pValueRobust.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << pValueRobust.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return pValueRobust[index] ;
}

double patPythonResults::getPValueRobust(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return pValueRobust[found->second] ;
}

int patPythonResults::  getFixed(unsigned int index) {
  if (index >= fixed.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << fixed.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return fixed[index] ;
}

int patPythonResults::  getFixed(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return fixed[found->second] ;
}

int patPythonResults::  getDistributed(unsigned int index) {
  if (index >= distributed.size()) {
    stringstream str ;
    str << "Index " << index << " out of range [0-" << distributed.size()-1 << "]" ;
    WARNING(str.str()) ;
    return double() ;
  }
  return distributed[index] ;
}

int patPythonResults::  getDistributed(const char* name) {
  patString theName(name) ;
  map<patString, unsigned int>::const_iterator found =
    paramIndex.find(theName) ;
  if (found == paramIndex.end()) {
    WARNING("Unknown parameter " << theName) ;
    return double() ;
  }
  return distributed[found->second] ;
}

