//-*-c++-*------------------------------------------------------------
//
// File name : bioArithPrint.h
// Author :    Michel Bierlaire
// Date :      Tue May 11 07:08:49 2010
//
//--------------------------------------------------------------------

#ifndef bioArithPrint_h
#define bioArithPrint_h

#include "bioExpression.h"
#include "bioLiteral.h"
#include "bioIteratorInfo.h"


/*!
Class implementing a node of the tree design to print the values of expressions
*/
class bioArithPrint : public bioExpression {

public:
  
  bioArithPrint(bioExpressionRepository* rep,
		patULong par,
		map<patString,patULong> theTerms,
		patString anIterator,
		patError*& err) ;

  ~bioArithPrint() ;

public:
  virtual bioExpression* getDerivative(patULong aLiteralId, 
				       patError*& err) const ;
  virtual patBoolean isSum() const  ;
  virtual patBoolean isProd() const  ;
  virtual patBoolean isSimulator() const ;
  void simulate(patError*& err) ;
  virtual patString getExpression(patError*& err) const ;
  virtual patString getOperatorName() const ;
  virtual patReal getValue(patBoolean prepareGradient,patError*& err) ;
  virtual bioArithPrint* getDeepCopy(bioExpressionRepository* rep, 
				     patError*& err) const ;
  virtual bioArithPrint* getShallowCopy(bioExpressionRepository* rep, 
				     patError*& err) const ;
  virtual patString getExpressionString() const ;
  virtual patULong getNumberOfOperations() const ;
  virtual patString getCppCode(patString prefix, patError*& err) ;
  virtual void generateGslFunctions(ostream& str_h,patString prefix,patError*& err) ;
  virtual bioFunctionAndDerivatives* getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) ;
  virtual void setVariables(const patVariables* x) ;
  //  virtual void setDraws(const vector< vector<patReal> >* d) ;
  virtual void setDraws(patReal** d) ;
  virtual void setSample(const bioSample* s) ;
  virtual void setCurrentSpan(bioIteratorSpan aSpan) ;
  virtual void setThreadSpan(bioIteratorSpan aSpan) ;
  virtual patString getSimpleCppCode(patError*& err)   ;
  virtual patBoolean dependsOf(patULong aLiteralId) const  ;
  virtual void forceCodeGeneration()  ;
  virtual patBoolean containsAnIterator() const ;
  virtual patBoolean containsAnIteratorOnRows() const ;
  virtual patBoolean containsAnIntegral() const ;
  virtual patBoolean containsASequence() const ;

  virtual patString getIncludeStatements(patString prefix) const ;
  virtual void simplifyZeros(patError*& err)  ;
  virtual void collectExpressionIds(set<patULong>* s) const  ;
  map<patString,vector<patReal> >* getSimulationResults()  ;
  virtual patString check(patError*& err) const  ;

private:
  map<patString,bioExpression*> theExpressions ;
  patString theIteratorName ;
  map<patString,patULong> theExpressionsId ;
  bioIteratorType theIteratorType ;

  map<patString,vector<patReal> > simulatedValues ;
};

#endif

