//-*-c++-*------------------------------------------------------------
//
// File name : bioArithSum.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue Jun 16 12:47:18  2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patError.h"

#include "trHessian.h"
#include "trParameters.h"

#include "bioRowIterator.h"
#include "bioMetaIterator.h"
#include "bioDrawIterator.h"
#include "bioLiteralRepository.h"
#include "bioParameters.h"
#include "bioRandomDraws.h"
#include "bioArithCompositeLiteral.h"

#include "bioIteratorInfoRepository.h"
#include "bioSample.h"

#include "bioArithSum.h"
#include "bioArithBinaryPlus.h"
#include "bioLiteralRepository.h"
#include "bioExpressionRepository.h"

/*!
*/
bioArithSum::bioArithSum(bioExpressionRepository* rep,
			 patULong par,
                         patULong left,
                         patString it,
			 patULong w,
			 patError*& err) 
  : bioArithIterator(rep, par,left,it,err),bhhh(NULL),weightId(w) {

  if (err != NULL) {
    WARNING(err->describe()) ;
  }
  
  weight = theRepository->getExpression(w) ;

  


}

bioArithSum::~bioArithSum() {}

patString bioArithSum::getOperatorName() const {
  return patString("Sum") ;
}

bioExpression* bioArithSum::getDerivative(patULong aLiteralId, patError*& err) const {

  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }
  
  bioExpression* leftResult = child->getDerivative(aLiteralId,err) ;

  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* result = new bioArithSum(theRepository,patBadId,leftResult->getId(),theIteratorName,weightId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }
  return result ;
}


bioArithSum* bioArithSum::getDeepCopy(bioExpressionRepository* rep,
				      patError*& err) const {
  bioExpression* leftClone(NULL) ;
  if (child != NULL) {
    leftClone = child->getDeepCopy(rep,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
  }
  bioArithSum* theNode = 
    new bioArithSum(rep,patBadId,leftClone->getId(),theIteratorName,weightId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNode ;
}

bioArithSum* bioArithSum::getShallowCopy(bioExpressionRepository* rep,
					 patError*& err) const {
  bioArithSum* theNode = 
    new bioArithSum(rep,patBadId,child->getId(),theIteratorName,weightId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNode ;
}



patBoolean bioArithSum::isSum() const {
  return patTRUE ;
}

patBoolean bioArithSum::isProd() const {
  return patFALSE ;
}

patString bioArithSum::getExpressionString() const {
  stringstream str ;
  str << "$S" ;
  str << theIteratorName ;
  if (child != NULL) {
    str << '{' << child->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}

patBoolean bioArithSum::isSumIterator() const {
  return patTRUE ;
}




patReal bioArithSum::getValue(patBoolean prepareGradient,patError*& err)  {

  if (child->getId() != childId) {
    stringstream str ;
    str << "Ids are different: " 
	<< child->getId() 
	<< " <> " 
	<< childId ;
    err = new patErrMiscError(str.str()) ;
    WARNING(err->describe()) ;
    return patReal() ;
  }
  
  
  if (theSample == NULL) {
    err = new patErrNullPointer("bioSample") ;
    WARNING(err->describe()) ;
    return patReal() ;
  }
  patReal result = 0.0 ;
  
  if(bhhh != NULL) {
    bhhh->setToZero() ;
  }

  if (theIteratorType == ROW) {
    bioRowIterator* theIter = theSample->createRowIterator(theCurrentSpan,
							   theThreadSpan,
							   err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal() ;
    }
    patULong nIter(0) ;
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      ++nIter ;
      child->setVariables(theIter->currentItem()) ;
      patReal expression = child->getValue(patFALSE,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return patReal() ;
      }
      if (weight != NULL && isTop()) {
	weight->setVariables(theIter->currentItem()) ;
	patReal w = weight->getValue(patFALSE,err) ;
	if (err != NULL) {
	  WARNING(err->describe()) ;
	  return patReal() ;
	}
	result += w * expression ;
      }
      else {
	result += expression ;
      }
    }
    DELETE_PTR(theIter) ;
    return result ;
  }
  else if (theIteratorType == META) {
    bioMetaIterator* theIter = theSample->createMetaIterator(theCurrentSpan,theThreadSpan,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal() ;
    }
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      child->setCurrentSpan(theIter->currentItem()) ;
      patReal expression = child->getValue(patFALSE,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return patReal() ;
      }
      result += expression ;
    }
    DELETE_PTR(theIter) ;
    return result ;
  }
  else if (theIteratorType == DRAW) {
    bioDrawIterator* theIter = bioRandomDraws::the()->createIterator() ;
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      child->setDraws(theIter->currentItem()) ;
      patReal expression = child->getValue(patFALSE,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return patReal() ;
      }
      result += expression ;
    }
    DELETE_PTR(theIter) ;
    patULong R = bioParameters::the()->getValueInt("NbrOfDraws",err)  ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal() ;
    }
    result /= patReal(R) ;

    return result ;
  }
  err = new patErrMiscError("Should never be reached") ;
  WARNING(err->describe()) ;
  return patReal() ;
}


trHessian* bioArithSum::getBhhh() {
  return bhhh ;
}


patULong bioArithSum::getNumberOfOperations() const {
  patError* err(NULL) ;
    
  patULong result = 0 ;
    
    
  if (theIteratorType == ROW) {
    bioRowIterator* theIter = theSample->createRowIterator(theCurrentSpan,theThreadSpan,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal() ;
    }
    patULong nIter(0) ;
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      ++nIter ;
      result += child->getNumberOfOperations() ;
    }
    DELETE_PTR(theIter) ;
    return result ;
  }
  else if (theIteratorType == META) {
    bioMetaIterator* theIter = theSample->createMetaIterator(theCurrentSpan,theThreadSpan,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patReal() ;
    }
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      child->setCurrentSpan(theIter->currentItem()) ;
      result += child->getNumberOfOperations() ;
    }
    DELETE_PTR(theIter) ;
    return result ;
  }
  else if (theIteratorType == DRAW) {
    bioDrawIterator* theIter = bioRandomDraws::the()->createIterator() ;
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      result += child->getNumberOfOperations() ;
    }
    DELETE_PTR(theIter) ;
    // There is one more division
    return 1+result ;
  }
  err = new patErrMiscError("Should never be reached") ;
  WARNING(err->describe()) ;
  return patReal() ;
}


bioFunctionAndDerivatives* bioArithSum::getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) {
  
  if (isTop() && bhhh == NULL) {
    trParameters p = bioParameters::the()->getTrParameters(err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    bhhh = new trHessian(p,literalIds.size()) ;
  }

  if (result.empty()) {
    result.resize(literalIds.size()) ;
  }
  patReal r = 0.0 ;
  
  vector<patReal> gradient(literalIds.size(),0.0) ;
  
  if(bhhh != NULL) {
    bhhh->setToZero() ;
  }
  
  if (theIteratorType == ROW) {
    bioRowIterator* theIter = theSample->createRowIterator(theCurrentSpan,
							   theThreadSpan,
							   err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL;
    }
    patULong nIter(0) ;
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      ++nIter ;
      child->setVariables(theIter->currentItem()) ;
      bioFunctionAndDerivatives* fg = child->getNumericalFunctionAndGradient(literalIds,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return NULL ;
      }
      patReal w ;
      if (weight != NULL && isTop()) {
	weight->setVariables(theIter->currentItem()) ;
	w = weight->getValue(patFALSE,err) ;
      }
      if ( r < patMaxReal) {
	if (fg->theFunction == patMaxReal) {
	  r = patMaxReal ;
	}
	else {
	  if (weight != NULL && isTop()) {
	    r += w * fg->theFunction ;
	  }
	  else {
	    r += fg->theFunction ;
	  }
	}
      }
      for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	if (weight != NULL && isTop()) {
	  gradient[i] += w * fg->theGradient[i] ;
	}
	else {
	  gradient[i] += fg->theGradient[i] ;
	}
      }
      if (bhhh != NULL) {
	for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	  for (patULong j = i ; j < literalIds.size() ; ++j) {
	    if (weight != NULL && isTop()) {

	      // This is the reason why the weight has to be handled
	      // separately. It only needs to be multipled once for
	      // the computation of the BHHH.
	      bhhh->addElement(i,j,w * fg->theGradient[i]*fg->theGradient[j],err) ;
	    }
	    else {
	      bhhh->addElement(i,j,fg->theGradient[i]*fg->theGradient[j],err) ;
	    }
	    if (err != NULL) {
	      WARNING(err->describe()) ;
	      return NULL ;
	    }
	  }
	}
      }
    }
    DELETE_PTR(theIter) ;


    result.theFunction = r ;
    for (patULong i = 0 ; i < literalIds.size() ; ++i) {
      result.theGradient[i] = gradient[i] ;
    }
    return &result ;
  }
  else if (theIteratorType == META) {
    bioMetaIterator* theIter = theSample->createMetaIterator(theCurrentSpan,theThreadSpan,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      child->setCurrentSpan(theIter->currentItem()) ;
      bioFunctionAndDerivatives* fg = child->getNumericalFunctionAndGradient(literalIds,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return NULL ;
      }
      r += fg->theFunction ;
      for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	gradient[i] += fg->theGradient[i] ;
      }
      if (bhhh != NULL) {
	for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	  for (patULong j = i ; j < literalIds.size() ; ++j) {
	    bhhh->addElement(i,j,fg->theGradient[i]*fg->theGradient[j],err) ;
	    if (err != NULL) {
	      WARNING(err->describe()) ;
	      return NULL ;
	    }
	  }
	}
      }
    }
    DELETE_PTR(theIter) ;
    result.theFunction = r ;
    for (patULong i = 0 ; i < literalIds.size() ; ++i) {
      result.theGradient[i] = gradient[i] ;
    }    
    return &result ;
  }
  else if (theIteratorType == DRAW) {
    bioDrawIterator* theIter = bioRandomDraws::the()->createIterator() ;
    patReal R = patReal(bioParameters::the()->getValueInt("NbrOfDraws",err))  ;
    for (theIter->first() ;
	 !theIter->isDone() ;
	 theIter->next()) {
      child->setDraws(theIter->currentItem()) ;
      bioFunctionAndDerivatives* fg = child->getNumericalFunctionAndGradient(literalIds,err) ;
      if (err != NULL) {
	WARNING(err->describe()) ;
	return NULL ;
      }
      if (performCheck) {
	r += fg->theFunction / R;
	for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	  gradient[i] += fg->theGradient[i] / R ;
	}
      }
      else {
	r += fg->theFunction ;
	for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	  gradient[i] += fg->theGradient[i] ;
	}
      }
    }
    DELETE_PTR(theIter) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return NULL ;
    }
    if (!performCheck) {
      r /= patReal(R) ;
    }
    result.theFunction = r ;
    if (performCheck) {
      for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	result.theGradient[i] = gradient[i] ;
      }   
    }
    else {
      for (patULong i = 0 ; i < literalIds.size() ; ++i) {
	result.theGradient[i] = gradient[i]/patReal(R) ;
      }   
    } 


    return &result ;
  }
  err = new patErrMiscError("Should never be reached") ;
  WARNING(err->describe()) ;
  return NULL ;
}
