//-*-c++-*------------------------------------------------------------
//
// File name : bioSample.h
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Fri Jun 19 10:05:49  2009
//
//--------------------------------------------------------------------

#ifndef bioSample_h
#define bioSample_h

#include <vector>
#include <map>
#include "patError.h"
#include "patVariables.h"
#include "bioIteratorSpan.h"
class bioMetaIterator ;
class bioRowIterator ;
class bioExpression ;
/*!

*/



/*!
   This object is in charge of the sample management.
 */
class bioSample {

public :

  bioSample(patString filename, patError*& err) ;
  
  ~bioSample() ;

  patULong size() const ;

  vector<patString> getHeaders() const ;

  patVariables* at(patULong index) ;

  patULong getIndexFromHeader(patString h, patError*& err) ;

  patString getFileName() ;
  
  patBoolean isEqual(bioSample s) ;
  
  void printSample() ;

  patULong getNumberOfColumns() const ;

  bioMetaIterator* createMetaIterator(bioIteratorSpan theSpan, bioIteratorSpan theThreadSpan, patError*& err) const ;

  bioRowIterator* createRowIterator(bioIteratorSpan theSpan, bioIteratorSpan theThreadSpan, patError*& err) const ;

  void prepare(bioExpression* exclude,
	       vector<pair<patString, bioExpression*> >* userExpressions, 
	       patError*& err) ;

  vector<patULong> getListOfIds(patString header, patError*& err) ;


private :

  void computeMapOfDatabase(patError*& err) ;
  void generatePythonHeaders(patError*& err) ;
  void readFile(bioExpression* exclude, 
		vector<pair<patString, bioExpression*> >* userExpressions, 
		patError*& err) ;

  void addObservationData(patVariables data) ;
  void printHeaders() ;

  vector<patString> headers ;
  vector<patVariables> samples ;

  patULong numberOfRealColumns ;
  
  patString fileName ;

};

#endif
